/* Copyright 2005-2008 Josh Clark and Global Moxie, LLC. All rights reserved.
   $Id: bm-config.js 3042 2008-03-31 13:56:04Z josh $
*/

Object.extend( BM.Valid.types,
    {   'section-slug' : {
            'function': function(fieldObj, option) {
                return BM.Valid.sectionSlug(fieldObj, option);
            }
        },
        'sitename' : {
            'function': function(fieldObj, option) {
                return BM.Valid.sitename(fieldObj, option);
            }
        }
    }
);

Object.extend( BM.Valid,
    {   'sitename' : function(field,option) {
            var id = $('site_id') ? $F('site_id')
                   : $('BMsite')  ? $F('BMsite')
                   :                BM.siteID;
            return BM.Config.validateUnique(field, option, 'bm-config.cgi', 'ajax-sitename-avail', id);
        },
        'sectionSlug' : function(field,option) {
            if ( !(field = $(field)) ) return;
            field.value = field.value.replace(/[^A-Za-z0-9\-\_]/g, '');
            var id = $F('section_id');
            return BM.Config.validateUnique(
              field, option, 'bm-siteconfig.cgi', 'ajax-sectionslug-avail', $F('section_id')
            );
        }
    }
);

// behaviors at bottom of file


/***************************************************************
    BM.Config
    Functions specific to establishing basic site/system settings
****************************************************************/

BM.Config = {
    swapDirView: function(evt, show, hide) {
        BM.FX.hide(hide,{effect:'slide',duration: 0.75});
        BM.FX.reveal(show);
        Event.stop(evt); return false;
    },
    updateHomeImg: function(ajax,statusObj) {
        BM.Config._updateImg(ajax,statusObj,'homepage');
    },
    updateHtmlImg: function(ajax,statusObj) {
        BM.Config._updateImg(ajax,statusObj,'html');
    },

    _updateImg: function(httpReq, statusObj, dirType) {
        BM.Ajax.clearStatus(statusObj);
        var ajaxJSON = BM.Ajax.parseJSON(httpReq);
        if (!ajaxJSON.data) {
            BM.FX.reveal('bmcp_' + dirType + 'ConfirmForm');
            if ( BM.Ajax.loginRequired(ajaxJSON) ) {
                BM.Ajax.clearStatus(statusObj);
            }
            else {
                BM.Ajax.showMessage(ajaxJSON, statusObj)
            }
            return false;
        }
        BM.FX.reveal('bmcp_' + dirType + 'CheckDiv');
        $('bmcp_' + dirType + 'Img').src = ajaxJSON.data.url;
        $('random_' + dirType).value = ajaxJSON.data.random;
    },

    confirmDir: function(dir, dirType, statusID) {
        if (!dir) {
            BM.Ajax.clearStatus(statusID);
            return false;
        }
        var statusObj = $(statusID);
        if (!statusObj) return false;
        if (!dirType) dirType = '';
        var script = document.location.href.match('/bm-start.cgi')
          ? '/bm-start.cgi' : '/bm-config.cgi';
        BM.Ajax.request(
            BM.moxiebin + script + BM.pathDiv + 'ajax-verify-dir/' + BM.siteID,
            { 'parameters' : 't=' + encodeURIComponent(dirType) + '&d='
                + encodeURIComponent(dir),
              'method' : 'post',
              'onSuccess': function(httpReq) {
                var ajaxJSON = BM.Ajax.parseJSON(httpReq);
                if ( BM.Ajax.loginRequired(ajaxJSON) ) {
                    BM.Ajax.clearStatus(statusObj);
                    return;
                }
                BM.Ajax.showMessage(ajaxJSON, statusObj)
              },
              'onFailure' : function() {  BM.Ajax.clearStatus(statusID) },
              'onLoading' : function() {BM.Ajax.inProgress(statusID)}
            }
        );
    },
    confirmHtmlDir: function(dir, fieldID) {
        var statusID = fieldID + 'AjaxStat'
        var statusObj = $(statusID);
        if (!statusObj) return false;
    
        var url = fieldID == 'html_dir' ? $F('html_url') : $F('homepage_url');
        if ( !BM.Ajax.isLocal(url) ) { // don't bother if not a local url
            BM.Ajax.clearStatus(statusObj);
            return;
        }
    
        var siteID = $('site_id') ? $F('site_id') : '';
        BM.Ajax.request(
            BM.moxiebin + '/bm-config.cgi' + BM.pathDiv + 'ajax-make-htmltrace/' + BM.siteID,
            { 'method': 'post',
              'postBody': "t=" + encodeURIComponent(fieldID)
                + "&d=" + encodeURIComponent(dir)
                + "&id=" + encodeURIComponent(siteID),
              'onSuccess': function(httpReq) {
                var ajaxJSON = BM.Ajax.parseJSON(httpReq);
                if ( BM.Ajax.loginRequired(ajaxJSON) ) {
                    BM.Ajax.clearStatus(statusObj);
                    return;
                }
                else {
                    BM.Config.checkTrace(ajaxJSON, fieldID, statusObj);
                }
              },
              'onFailure' : function() {  BM.Ajax.clearStatus(statusID) },
              'onLoading' : function() {BM.Ajax.inProgress(statusID)}
            }
        );
    },
    checkTrace: function(ajaxJSON, fieldID, statusObj) {
        if (ajaxJSON.valid != 'OK') { // could not create trace
            BM.Ajax.showMessage(ajaxJSON, statusObj);
            return;
        }
        
        //confirm that url exists
        var url = fieldID == 'html_dir' ? $F('html_url') : $F('homepage_url');
        url = url + '/bm_' + fieldID + '.txt?' + BM.random();
        var ajax = BM.Ajax.request(
            url,
            { 'method': 'get', // ie6 doesn't like head request
              'onComplete': function(ajax) {
                BM.Config.removeTrace(ajax, fieldID, statusObj)
              }
            }
        );
        if (!ajax) { //bail out; probably not the same domain as bm
            BM.Config.removeTrace( {bailout:1},fieldID,statusObj );
        }
    },

    removeTrace: function(ajax, fieldID, statusObj) {  
        if (ajax.status == 200) {
            statusObj.innerHTML='<span class="bm_ajaxOK">'
              + BM.Lang._('Config:Directory confirmed') + '</span>';
        }
        else if (ajax.bailout) {
            BM.Ajax.clearStatus(statusObj);
        }
        else {
            var message =
                ajax.status == 404 ? BM.Lang._('Config:Directory not found')
              : BM.Lang._('Config:Directory trace error') + ajax.statusText;
            statusObj.innerHTML='<span class="bm_ajaxNotOK">'+message+'</span>';
        }
        var force_check = statusObj.innerHTML; // help update firefox see update
        var siteID = $('site_id') ? $F('site_id')
               : $('BMsite')  ? $F('BMsite')
               :                '';
        BM.Ajax.request(
            BM.moxiebin + '/bm-config.cgi' + BM.pathDiv + 'ajax-remove-htmltrace/' + BM.siteID,
            { 'method': 'post',
              'postBody': "t=" + encodeURIComponent(fieldID) + "&d="
                + encodeURIComponent($F(fieldID)) + "&id=" + encodeURIComponent(siteID)
            }
        );
    },
    validateUnique: function (field, option, script, runmode, id) {
        if ( !(field = $(field)) ) return;
        if (field.value.match(/\n\r/)) return false;
        if (typeof option != 'object') option = {};
        if (!option.submission) BM.Field.ajaxValidate(field, script, runmode, id);
        return true;
    }
};

BM.Behavior.register({
    '#bmcp_homepageYes:click' : function(e,evt) {
      BM.Config.swapDirView(evt, 'bmcp_htmlConfirm', 'bmcp_homepageConfirm');
    },
    '#bmcp_homepageNo:click' : function(e,evt) {
      BM.Config.swapDirView(evt, 'bmcp_homepageConfirmForm', 'bmcp_homepageCheckDiv');
    },
    '#bmcp_homepage_form:submit' : function(e,evt) {
        if (!BM.Valid.validateForm(e)){ Event.stop(evt); return false; }
        BM.Ajax.submitForm(e, 'bmcp_homepageStatus', 
          {onsuccess:BM.Config.updateHomeImg} );
        var old_dir=$('old_homepage_dir');
        old_dir.value=$F('homepage_dir');
        BM.FX.hide('bmcp_homepageConfirmForm');
        Event.stop(evt); return false;
    },

    '#bmcp_htmlYes:click' : function(e,evt) { //allow to continue
        BM.FX.hide('bmcp_htmlConfirm',{effect:'slide',duration: 0.75});
    },
    '#bmcp_htmlNo:click' : function(e,evt) {
      BM.Config.swapDirView(evt, 'bmcp_htmlConfirmForm', 'bmcp_htmlCheckDiv');
    },
    '#bmcp_html_form:submit' : function(e,evt) {
        if (!BM.Valid.validateForm(e)){ Event.stop(evt); return false; }
        BM.Ajax.submitForm(e, 'bmcp_htmlStatus', 
          {onsuccess:BM.Config.updateHtmlImg} );
        var old_dir=$('old_html_dir');
        old_dir.value=$F('old_html_dir');
        BM.FX.hide('bmcp_htmlConfirmForm');
        Event.stop(evt); return false;
    }
});



/***************************************************************
  BM.TimeOffset
  Initialization and handling of time offset fields
****************************************************************/

BM.TimeOffset = {
    setMsg: function(sign, hour, minute) {
        BM.TimeOffset.msg = BM.Lang._('TimeOffset:gmt_help') + sign
          + hour + ':' + minute + '.';
    },
    init: function(field) {
        if (!(field = $(field))) return;
        var minutes = ( new Date().getTimezoneOffset() ) * -1;
        var sign;
        if (minutes < 0) {
            sign = '-';
            minutes = minutes * -1;
        }
        else {
            sign = '+';
        }
        var hours = Math.floor(minutes/60);
        minutes = minutes - (hours * 60);
        minutes = Math.round(minutes/15) * 15; // round to nearest 15
        if (minutes < 10) minutes = '0' + minutes;
    
        if (field.value == '') {
            var signSelect = $(field.id + '-sign');
            if (signSelect) signSelect.selectedIndex = sign == '+' ? 0 : 1;
            var hourSelect = $(field.id + '-hour');
            if (hourSelect) hourSelect.selectedIndex = hours;
            var minuteSelect = $(field.id + '-minute');
            if (minuteSelect) minuteSelect.selectedIndex = minutes/15;
        }
        BM.TimeOffset.setMsg(sign, hours, minutes);
        BM.TimeOffset.recalc(field.id);
    },
    recalc: function(id) {
        var signSelect = $(id + '-sign');
        var sign = 1;
        if (signSelect && signSelect.selectedIndex > 0) sign = -1;
        var hourSelect = $(id + '-hour');
        var hour = 0;
        if (hourSelect) hour = hourSelect.selectedIndex;
        var minuteSelect = $(id + '-minute');
        var minute;
        if (minuteSelect) minute = minuteSelect.selectedIndex * 15;
        minute = sign * (minute + (hour * 60));
        var time_now = new Date;
        var offset_Time = new Date(time_now.getTime() + (minute * 60 * 1000));
        var adjHour = offset_Time.getUTCHours();
        var ampm;
        if (adjHour > 12) {
            ampm = 'pm';
            adjHour = adjHour - 12;
        }
        else if (adjHour == 12) {
            ampm = 'pm';
        }
        else {
            ampm = 'am';
        }
        var adjMin = offset_Time.getUTCMinutes();
        if (adjMin < 10) adjMin = '0' + adjMin;
        var statusObj = $(id + '-info');
        statusObj.innerHTML = BM.TimeOffset.msg + '<br />'
          + BM.Lang._('TimeOffset:calculated_time')
          + '<span id="' + id + '-adj_time" class="bmcp_adj_time">'
          + adjHour + ':' + adjMin + ampm + '</span>.'
        BM.FX.flash(id + '-adj_time',
          {startcolor: '#99ff99', endcolor: '#ffffff', restorecolor:'#ffffff'}
        );
    }
};
