/* $Id: bm-css.js 3068 2008-05-28 16:11:12Z josh $ */

/*   Add toJSONString to BM object
****************************************************************/
(function () {
    var m = {
            '\b': '\\b',
            '\t': '\\t',
            '\n': '\\n',
            '\f': '\\f',
            '\r': '\\r',
            '"' : '\\"',
            '\\': '\\\\'
        },
        s = {
            array: function (x) {
                var a = ['['], b, f, i, l = x.length, v;
                for (i = 0; i < l; i += 1) {
                    v = x[i];
                    f = s[typeof v];
                    if (f) {
                        v = f(v);
                        if (typeof v == 'string') {
                            if (b) {
                                a[a.length] = ',';
                            }
                            a[a.length] = v;
                            b = true;
                        }
                    }
                }
                a[a.length] = ']';
                return a.join('');
            },
            'boolean': function (x) {
                return String(x);
            },
            'null': function (x) {
                return "null";
            },
            number: function (x) {
                return isFinite(x) ? String(x) : 'null';
            },
            object: function (x) {
                if (x) {
                    if (x instanceof Array) {
                        return s.array(x);
                    }
                    var a = ['{'], b, f, i, v;
                    for (i in x) {
                        v = x[i];
                        f = s[typeof v];
                        if (f) {
                            v = f(v);
                            if (typeof v == 'string') {
                                if (b) {
                                    a[a.length] = ',';
                                }
                                a.push(s.string(i), ':', v);
                                b = true;
                            }
                        }
                    }
                    a[a.length] = '}';
                    return a.join('');
                }
                return 'null';
            },
            string: function (x) {
                if (/["\\\x00-\x1f]/.test(x)) {
/* josh: x.replace to prototype's x.gsub to help old versions of safari
                    x = x.replace(/([\x00-\x1f\\"])/g, function(a, b) {
*/
                    x = x.gsub(/[\x00-\x1f\\"]/, function(match) {
                        var b = match[0];
//end patch
                        var c = m[b];
                        if (c) {
                            return c;
                        }
                        c = b.charCodeAt();
                        return '\\u00' +
                            Math.floor(c / 16).toString(16) +
                            (c % 16).toString(16);
                    });
                }
                return '"' + x + '"';
            }
        };

    BM.toJSONString = s.object;
})();


/***************************************************************
    ADD EVENTS AND BEHAVIORS
****************************************************************/

BM.addLoadEvent( function() { //add post-validation callback to handle form
    $('bmcpCSSForm').onvalidate = function() {
        try {
            BM.CSSedit.singleton().saveStyles($('bmcpCSSForm'));
            BM.confirmLeave = false; // okay to leave after save
        }
        catch (e) {
            var msg = BM.Lang._('BM:Unexpected error');
            alert( msg + "\n\n" + e.name + ":\n" + e.message );
        }
        return false;
    };
} );

window.onload = function() { // have to wait til the iframe loads
    BM.CSSedit.singleton();
    var cUpdater = function(f){ BM.CSSedit.singleton().changeCSSByField(f); };
    new BM.ColorPicker('BMCSS_color',cUpdater);
    new BM.ColorPicker('BMCSS_border-color',cUpdater);
    new BM.ColorPicker('BMCSS_background-color',cUpdater);
}

/* Remaining behaviors are at the end of the file */



/***************************************************************/
    BM.CSSedit = Class.create();

/*  Manage CSS editing tasks
****************************************************************/

BM.CSSedit.singleton = function() {
    if (!BM.CSSedit.__singleton) {
        BM.CSSedit.__singleton = new BM.CSSedit('bmcpPreviewCSS');
    }
    return BM.CSSedit.__singleton;
}
BM.CSSedit.rules = {}; // container for json css rules
if (!BM.isSafari) {
    BM.CSSedit.hilite = $H({
        'borderColor' : 'red',
        'borderStyle' : 'solid',
        'borderWidth' : '2px',
        'backgroundColor' : '#fcc'
    });
}
else {
    BM.CSSedit.hilite = $H({ //safari doesn't cope well with border styles
        'backgroundColor' : '#fcc'
    });
}

// parse and prompt rules for special fields
BM.CSSedit.getLength = function(field) {
    if ( !( field = $(field) ) ) return;
    var base = field.id.replace(/___.*/,"");
    if ( base != field.id ) field = $(base);
    field.value = field.value.replace(/[^\-\d\.]/g, "")
    return field.value != '' ? field.value + $F(field.id + '___UNIT') : '';
};
BM.CSSedit.applyLength = function(attr,value) {
    if (typeof value == 'undefined' || value == null) value = '';
    var match = value.strip().match(/^(\-?[\d.]*)\s*(\D*)/);
    var unit;
    if (match) value = match[1], unit = match[2];
    if (typeof unit == 'undefined' || !unit) unit = 'px';
    $('BMCSS_' + attr).value = value;
    $('BMCSS_' + attr + '___UNIT').value = unit;
};
BM.CSSedit.applyFontStyle = function(attr,value) {
    if (typeof value == 'undefined' || value == null) value = '';
    BM.Tritoggle.setValue('BMCSS_' + attr, value);
};
BM.CSSedit.applyColor = function(attr,value) {
    if (typeof value == 'undefined' || value == null) value = '';
    var field = $('BMCSS_'+attr);
    field.value = value;
    if (field.updateSample) field.updateSample();
};
BM.CSSedit.clickValueButton = function(e, event) {
    var cid = e.id;
    var base = cid.replace(/___.*$/, '');
    BM.CSSedit.singleton().changeCSSByField($(base));
}

// the CSSedit object
BM.CSSedit.prototype = {
    initialize: function(iframe) {
        this.iframe = $(iframe);
        this.idoc = BM.iframeDoc(this.iframe);
        this.changePreview('BM_PAGE_PREVIEW');

        this.rules = BM.CSSedit.rules;
        this.name = iframe.id;
        this.selector = $F('selector');
        if (!this.initStyles()) return;

        // has to come after initStyles or can get crazy loop of no published
        // pages are present
        if (!BM.iframeInitialized) {
            //ie doesn't know iframe.onload, use more robust event observer;
            //instead of creating new object, initialize current one, or
            //run into memory leak issues in ie
            //Meanwhile, safari seems to have trouble with adding an
            //event observer, so use onload.
            var onload=function(){BM.CSSedit.singleton().initialize(iframe)};
            if (BM.isSafari) this.iframe.onload = onload;
            else Event.observe(this.iframe, 'load', onload);
            BM.iframeInitialized = 1;
        }

        //disable links;we lose control of iframe if go out of same domain
        var doNothing = function(){return false;};
        this.$$('a').each(function(link){link.onclick=doNothing});
        this.updateSelector(); // sets and highlights selector
    },
    initStyles : function() {
        this.stylesheet = null;
        var cssLink = this.idoc.getElementById('BM_CUSTOM_SHEET');
        if (!cssLink) {
            if ( this.idoc.body.innerHTML.indexOf('BM_CURRENT_SITE') >= 0 ) {
                return false; //big medium error message, let it through
            }
            this.idoc.body.innerHTML = '';
            BM.alert(
                'The style editor works only with pages '
                + 'generated from templates containing the '
                + '<code>&lt;%htmlhead%&gt;</code> widget. Please revise '
                + "the site's templates and try again.");
            return false;
        }
        cssLink.disabled = true; //turn off real custom sheet

        //working sheet is guaranteed to be the last one...
        this.stylesheet =
          this.idoc.styleSheets[ this.idoc.styleSheets.length - 1];

        this.w3c = (typeof this.stylesheet.cssRules != "undefined");
        var sh = this.stylesheet;

        //clear all rules from style sheet
        var ruleList = this.w3c ? sh.cssRules : sh.rules;
        var items = ruleList.length;
        for(var i=0;i<items;i++) {
            if (this.w3c) sh.deleteRule(0);
            else sh.removeRule(0);
        }

        //fill the style sheet with the json rules
        this.styleMap = {};
        var css = this;
        if (BM.isSafari) { // safari won't redraw without adding a new style
            css.updateElement('body',$H({'color':'inherit'}));
            this.styleMap = {};
            sh.deleteRule(0);
        }
        $H(this.rules).each(function(rule){    // [selector, {styles}]
            var styles = $H(rule[1]);
            css.updateElement(rule[0],styles);
        });
        return true;
    },
    updateSelector : function() {
        var newSel = $F('selector');
        if (newSel != this.selector) this.removeHighlight(this.selector);
        this.selector = newSel;
        var thisRule = this.selectorRules();
        var self = this;
        $H(this.ruleMap).keys().each(function(attr){
            var prompter = typeof self.prompters[attr] != 'undefined'
                         ? self.prompters[attr]
                         : self.prompters.defaultPrompter;
            var value =
              ( typeof thisRule[attr] == 'undefined' || thisRule[attr] == null )
              ? '' : thisRule[attr].toString().strip();
            prompter(attr,value);
        });
        var legend = $('bmfs_cssrules').getElementsByTagName('LEGEND')[0];
        legend.innerHTML = 'CSS: ' + this.selector;
        this.updateHighlight();
    },
    $$ : function() { //iframe-specific $$
        var idoc = this.idoc;
        if (idoc.defaultView && idoc.defaultView.$$){ //access iframe directly
            return idoc.defaultView.$$( arguments[0] );
        }

        // ugh, have to access iframe elements through parent window 
        var expr = arguments[0].strip();
        try {
            var selector = new Selector(expr);
            return selector.findElements(idoc);
        }
        catch (e) { // firefox3 doesn't like xpath on iframe, trick it
/* after upgrade to prototype 1.6, this is better than what's below:
BM.CSSedit.frame$$ ||= Class.create(Selector, {
    shouldUseXPath : function() { return false; }
});
var frameselector = new BM.CSSedit.frame$$('li.foo');
return frameselector.findElements(iDoc);
*/
            var tempXpath = Prototype.BrowserFeatures.XPath;
            Prototype.BrowserFeatures.XPath = null;
            delete Selector._cache[expr];
            var selector = new Selector(expr);
            var results = selector.findElements(idoc);
            delete Selector._cache[expr];
            Prototype.BrowserFeatures.XPath = tempXpath;
            return results;
        }
    },
    updateElement : function(selector, styles) {
        // styles is hash, or array of arrays: [ [attribute, value] ... ]

        if (selector.indexOf(',') > 0) { //ie doesn't like commas
            var self = this;
            selector.split(/,/).each(function(indiv){
                self.updateElement(indiv, styles);
            });
            return;
        }

        if (!this.styleMap[selector]) { //add a style for this selector
            var index = 0;
            
            // :visited links have to appear after:link
            // and :hover after :visited
            if ( selector.match(/:visited$/) ) {
                var base = selector.replace(/visited$/, "");
                if (this.styleMap[base+"hover"]) {
                    index = this.lookupStyleIndex(base+"hover");
                }
                else {
                    index = this.nextStyleIndex();
                }
            }
            else if ( selector.match(/:hover$/) ) {
                index = this.nextStyleIndex();
            }

            if (this.w3c) {
                this.stylesheet.insertRule(selector + "{}", index);
                this.styleMap[selector] = this.stylesheet.cssRules[index];
            }
            else { // ie doesn't like empty value, give something innocuous
                this.stylesheet.addRule(
                  selector, "voice-family:inherit", index
                );
                this.styleMap[selector] = this.stylesheet.rules[index];
            }
        }
        var rule = this.styleMap[selector];
        var css = this;
        styles.each(function(style){        // [attr, value]
            var attr = css.ruleMap[style[0]] || style[0];
            if (style[1] == null) style[1] = ''; //ie chokes on null
            try {
                rule.style[attr] = style[1];
            }
            catch (e) { // probably some bad value in the argument, ignore it
                rule.style[attr] = '';
            }
        });
    },
    lookupStyleIndex : function(selector) {
        var rules = this.w3c ? this.stylesheet.cssRules : this.stylesheet.rules;
        for (var i=0; i<rules.length; i++) {
            //ie6 screws with the case of the selector
            if (rules[i].selectorText.toLowerCase() == selector.toLowerCase()) return i;
        }
    },
    nextStyleIndex : function() {
        var rules = this.w3c ? this.stylesheet.cssRules : this.stylesheet.rules;
        return rules.length;
    },
    changeCSSByField : function(field) {
        var thisRule = this.selectorRules();
        var attr = field.id.replace(/^BMCSS_(.*)/, "$1");
        attr = attr.replace(/__.*$/, '');
        var parser = typeof this.parsers[attr] != 'undefined'
                   ? this.parsers[attr]
                   : this.parsers.defaultParser;
        var value = parser(field);
        thisRule[attr] = value == '' ? null : value;
        this.updateElement(this.selector, $H(thisRule));
        BM.confirmLeave = true; // mark field as dirty
    },
    selectorRules : function(selector) {
        if (!selector) selector = this.selector;
        if (!this.rules[selector]) this.rules[selector] = {};
        return this.rules[selector];
    },
    updateHighlight : function(selector) {
        if (!selector) selector = this.selector;
        if (selector.indexOf(',') > 0) {
            var self = this;
            selector.split(/,/).each(self.updateHighlight.bind(self));
            return;
        }
        var noPseudo = selector.replace(/:[^\s,]+/g, "");
        if ($F('highlight_element_1')) {
            var rules = BM.CSSedit.hilite;
            this.$$(noPseudo).each( function(element) {
                rules.each(function(rule) { element.style[rule[0]]=rule[1] });
            });
        }
        else {
            this.removeHighlight(noPseudo);
        }
    },
    removeHighlight : function(selector) {
        if (selector.indexOf(',') > 0) {
            var self = this;
            selector.split(/,/).each(self.removeHighlight.bind(self));
            return;
        }
        var noPseudo = selector.replace(/:[^\s,]+/g, "");
        var rules = BM.CSSedit.hilite;
        this.$$(noPseudo).each( function(element) {
            rules.keys().each(function(attr){ element.style[attr] = "" });
        });
    },
    saveStyles : function(form) {
        var iframe = this.iframe;
        iframe.style.visibility="hidden";
        var modal = BM.alert('<div id="bmcpCSSstatus"></div>');
        var statusObj = $('bmcpCSSstatus');
        var onsuccess = function(httpReq, update) {
            var ajaxJSON = BM.Ajax.parseJSON(httpReq);
            if (BM.Ajax.loginRequired(ajaxJSON)) {
                modal.remove();
                iframe.style.visibility = "visible";
            }
            else if (ajaxJSON.html) {
                BM.Ajax.showMessage(ajaxJSON, statusObj);
                setTimeout(function(){
                    modal.remove();
                    iframe.style.visibility = "visible";
                  },800);
            }
            else {
                iframe.style.visibility = "visible";
                BM.Ajax.showMessage(ajaxJSON, statusObj);
            }
        };
        var param = 'styles=' + encodeURIComponent(BM.toJSONString(this.rules));
        BM.Ajax.request(
            form.action,
            { 'method' : 'post',
              'postBody' : param,
              'parameters' : param,
              'onLoading' : function() { BM.Ajax.inProgress(statusObj) },
              'onSuccess' : function(httpReq) { onsuccess(httpReq, statusObj) },
              'onFailure' : function(httpReq) { BM.Ajax.error(httpReq, statusObj) }
            }
        );
    },
    changePreview : function(select) {
        if ( !( select = $(select) ) ) return;
        var url = $F(select);
        if (url != this.iframe.src) {
            this.idoc.body.style.margin='125px auto';
            BM.Ajax.inProgress(this.idoc.body);
            this.iframe.src = url;
        }
    },
    ruleMap : {
        "color"       : "color",
        "font-family" : "fontFamily",
        "font-size"   : "fontSize",
        "font-weight" : "fontWeight",
        "background-color" : "backgroundColor",
        "background-image" : "backgroundImage",
        "background-repeat" : "backgroundRepeat",
        "border-color" : "borderColor",
        "border-width" : "borderWidth",
        "border-style" : "borderStyle",
        "line-height"  : "lineHeight",
        "margin-top" : "marginTop",
        "margin-right" : "marginRight",
        "margin-left" : "marginLeft",
        "margin-bottom" : "marginBottom",
        "padding-top" : "paddingTop",
        "padding-bottom" : "paddingBottom",
        "padding-right" : "paddingRight",
        "padding-left" : "paddingLeft",
        "text-align" : "textAlign",
        "text-decoration" : "textDecoration",
        "font-style" : "fontStyle",
        "width" : "width",
        "height" : "height"
    },
    parsers : {
        'defaultParser' : function(field) { return field.value },
        'font-family' : function(select) {
            if (!select) select = 'BMCSS_font-family';
            if ( !( select = $(select) ) ) return;
            var base = select.id.replace(/___\d$/, "");
            var numfields = $F(base);
            var fonts = [];
            for (var i = 1; i <= numfields; i++) {
                var select = $(base + '___' + i);
                var font = select.value.strip();
                if (font.indexOf(' ') >= 0) font = '"' + font + '"';
                if (font == '---') font = '';
                select.style.fontFamily = font;
                if (font) fonts.push(font.toLowerCase());
            }
            return fonts.join(',');
        },
        'line-height' : BM.CSSedit.getLength,
        'height' : BM.CSSedit.getLength,
        'width' : BM.CSSedit.getLength,
        'border-width' : BM.CSSedit.getLength,
        'margin-top' : BM.CSSedit.getLength,
        'margin-left' : BM.CSSedit.getLength,
        'margin-right' : BM.CSSedit.getLength,
        'margin-bottom' : BM.CSSedit.getLength,
        'padding-top' : BM.CSSedit.getLength,
        'padding-left' : BM.CSSedit.getLength,
        'padding-right' : BM.CSSedit.getLength,
        'padding-bottom' : BM.CSSedit.getLength,
        'font-size' : function(field) {
            var base = 'BMCSS_font-size';
            var value;
            if ($(base+'___RKEY').checked) return $(base+'__KEYWORD').value;
            else return BM.CSSedit.getLength(base+'__SIZE');
        },
        'background-image' : function(field) {
            var value = field.value.strip();
            if (value == 'http://') return '';
            else return 'url(' + value + ')';
        }
    },
    prompters : {
        'defaultPrompter' : function(attr,value) {
            $('BMCSS_' + attr).value = value;
        },
        'font-family' : function(attr,value) {
            var fonts = value.split(/,\s*/).map(function(f){
                return f.replace(/"/g,"").strip().toLowerCase();
            });
            var base = 'BMCSS_font-family';
            var numfields = $F(base);
            for (var i = 1; i <= numfields; i++) {
                var select = $(base + '___' + i);
                var index = 0;
                if (typeof fonts[i-1] != 'undefined') {
                    var matchOpt = $A(select.options).detect(function(opt){
                        return opt.value.toLowerCase() == fonts[i-1];
                    });
                    var index = matchOpt ? matchOpt.index : 0;
                }
                select.selectedIndex = index;
                var thisFont = select.value;
                if(thisFont == '---') thisFont = '';
                select.style.fontFamily = thisFont;
            }
        },
        'font-size' : function(attr,value) {
            value = value.toLowerCase();
            var base = 'BMCSS_' + attr;
            var keyword = (
                value == 'small'
             || value == 'medium'
             || value == 'large'
             || value == 'x-small'
             || value == 'x-large'
             || value == 'xx-large'
             || value == 'xx-small' ) ? value : '';
            var size = keyword ? '' : value;
            BM.CSSedit.applyLength(attr + '__SIZE', size);
            $(base+'__KEYWORD').value=keyword;
            if(size != '') {
                $(base + '___RSIZE').checked = true;
                Element.show(base+'___SIZETOG');
                Element.hide(base+'___KEYWORDTOG');
            }
            else {
                $(base + '___RKEY').checked = true;
                Element.hide(base+'___SIZETOG');
                Element.show(base+'___KEYWORDTOG');
            }
        },
        'text-align' : function(attr,value) {
            $('BMCSS_text-align').value=value;
            BM.FormWidget.setImageButtons('BMCSS_text-align',value,'');
        },
        'background-image' : function(attr,value) {
            if (!value) value = '';
            var reg = /^url\((.+?)\)/;
            value = value.strip().replace(reg, "$1");
            if (!value) value = 'http://';
            $('BMCSS_background-image').value=value;
        },
        'line-height' : BM.CSSedit.applyLength,
        'width' : BM.CSSedit.applyLength,
        'height' : BM.CSSedit.applyLength,
        'border-width' : BM.CSSedit.applyLength,
        'margin-top' : BM.CSSedit.applyLength,
        'margin-left' : BM.CSSedit.applyLength,
        'margin-right' : BM.CSSedit.applyLength,
        'margin-bottom' : BM.CSSedit.applyLength,
        'padding-top' : BM.CSSedit.applyLength,
        'padding-left' : BM.CSSedit.applyLength,
        'padding-right' : BM.CSSedit.applyLength,
        'padding-bottom' : BM.CSSedit.applyLength,
        'font-weight' : BM.CSSedit.applyFontStyle,
        'font-style' : BM.CSSedit.applyFontStyle,
        'text-decoration' : BM.CSSedit.applyFontStyle,
        'color' : BM.CSSedit.applyColor,
        'background-color' : BM.CSSedit.applyColor,
        'border-color' : BM.CSSedit.applyColor
    }
};


/***************************************************************/
    BM.ColorPicker = Class.create();

/*  Color picker
****************************************************************/

BM.ColorPicker.prototype = {
    initialize: function(field,onupdate) {
        if ( !(this.field = $(field) ) ) return;
        if ( typeof onupdate == 'function' ) this.onupdate = onupdate;

        this.sample = Builder.node(
            'button',
            { type      : 'button',
              className : 'bm_cpicker',
              id        : this.field.id + '___BMCPICKSAMPLE'
            }
        );
        this.transparentBg = 'url(' + BM.imagedir + '/bm_tritoggle_default.gif)';
        this.sample.innerHTML='&nbsp;';
        this.sample.style.backgroundColor = this.field.value;
        if (!this.field.value) this.sample.style.backgroundImage = this.transparentBg;
        this.field.parentNode.appendChild(this.sample);

        //set to absolute, but position only when sample is clicked, in
        //case we're adding it to an undisplayed portion of the page
        this.picker = Builder.node(
            'div',
            { style : 'position:absolute;display:none;'
                      + 'padding:11px;background-image:url('
                      + BM.imagedir + '/bm_colorscale_back.png);'
                      + 'background-repeat:no-repeat;z-index:1000;',
              id    : this.field.id + '___BMCPICKER'
            }
        );
        this.image = Builder.node(
            'img',
            { src   : BM.imagedir + '/bm_colorscale.png',
              alt   : '',
              style : 'cursor:crosshair;'
            }
        );        
        this.picker.appendChild(this.image);
        document.body.appendChild(this.picker);

        if (BM.isIE) { // slip in the iframe to protect against selects
            this.ieFrame = Builder.node('iframe', {
                src : document.location.protocol == 'https:'
                      ? '//0' : "javascript:function(){return false;}",
                style : 'position:absolute;width:132px;height:214px;'
                      + 'top:0;left:0;display:none;z-index:999;',
                scrolling: 'no',
                frameBorder: '0'
            });
            document.body.insertBefore(this.ieFrame,this.picker);
            Element.setOpacity(this.ieFrame,0);
        }

        Event.observe(document.body,'click',this.clickCheck.bind(this),false);
        
        var self = this;
        this.field.updateSample = this.field.onchange = self.updateDisplay.bind(self);
        this.sample.onclick = function(){
            this.blur();
            if (Element.visible(self.picker)) self.hideColorPicker();
            else self.showColorPicker();
            return false;
        };
        this.image.onmousemove = self.getColor.bind(self);
        this.image.onclick = function(event) {
            self.getColor(event);
            self.hideColorPicker();
        }
    },
    getColor : function(event) {
        if (typeof(event) == 'undefined') event = window.event;
        var x = Event.pointerX(event) - this.posx;
        var y = Event.pointerY(event) - this.posy;

        var color = null;
        if (x < 0) { // left console
            if (y < 11) {
                this.updateDisplay('');
            }
            else {
                for (var i=19; i<=115 ; i+=8) {
                    if (y<i) {
                        this.updateDisplay(this.consoleMap[i.toString()]);
                        break;
                    }
                }
            }
            return false;
        }

        var rmax = 0;
        var gmax = 0;
        var bmax = 0;
        if (y <= 32) {
            rmax = 255;
            gmax = (y / 32.0) * 255;
            bmax = 0;
        }
        else if (y <= 64) {
            y = y - 32;
            rmax = 255 - (y / 32.0) * 255;
            gmax = 255;
            bmax = 0;
        }
        else if (y <= 96) {
            y = y - 64;
            rmax = 0;
            gmax = 255;
            bmax = (y / 32.0) * 255;
        }
        else if (y <= 128) {
            y = y - 96;
            rmax = 0;
            gmax = 255 - (y / 32.0) * 255;
            bmax = 255;
        }
        else if (y <= 160) {
            y = y - 128;
            rmax = (y / 32.0) * 255;
            gmax = 0;
            bmax = 255;
        }
        else {
            y = y - 160;
            rmax = 255;
            gmax = 0;
            bmax = 255 - (y / 32.0) * 255;
        };
    
        var r;
        var g;
        var b;
        if (x <= 50) {
            r = Math.abs(Math.floor(rmax * x / 50.0));
            g = Math.abs(Math.floor(gmax * x / 50.0));
            b = Math.abs(Math.floor(bmax * x / 50.0));
        }
        else {
            x -= 50;
            r = Math.abs(Math.floor(rmax + (x / 50.0) * (255 - rmax)));
            g = Math.abs(Math.floor(gmax + (x / 50.0) * (255 - gmax)));
            b = Math.abs(Math.floor(bmax + (x / 50.0) * (255 - bmax)));
        }
        var color = '#' + r.toColorPart() + g.toColorPart() + b.toColorPart();
        this.updateDisplay(color);
        return false;
    },
    updateDisplay : function(color) {
        if (typeof color == 'object' || typeof color == 'undefined') {
            color=this.field.value;
        }
        else {
            this.field.value = color;
        }
        
        if (!color) {
            this.sample.style.backgroundImage = this.transparentBg;
        }
        else {
            this.sample.style.backgroundColor = color;
            this.sample.style.backgroundImage = 'none';
        }
        if (this.onupdate) this.onupdate(this.field);
    },
    clickCheck : function(event) {
        var cid = (event.target || event.srcElement).id;
        if ( cid != this.picker.id
          && cid != this.sample.id
          && cid != this.field.id
        ) {
            this.hideColorPicker();
        }
    },
    positionColorPicker : function() {
        var pos = Position.cumulativeOffset(this.sample);
        this.posx = pos[0] + this.sample.offsetWidth + 5;
        this.posy = pos[1];
        this.picker.style.left = this.posx + 'px';
        this.picker.style.top = this.posy + 'px';
        if (this.ieFrame) {
            this.ieFrame.style.left = this.posx + 'px';
            this.ieFrame.style.top = this.posy + 'px';
        }
        //change to set to top left of image, accounting for pad & left console
        this.posx += 21;
        this.posy += 11;
    },
    showColorPicker : function() {
        this.positionColorPicker();
        if(this.ieFrame) Element.show(this.ieFrame);
        BM.FX.reveal(this.picker);
    },
    hideColorPicker : function() {
        Element.hide(this.picker);
        if(this.ieFrame) Element.hide(this.ieFrame);
    },
    consoleMap : {
        '19' : 'transparent',
        '27' : '#000000',
        '35' : '#333333',
        '43' : '#666666',
        '51' : '#999999',
        '59' : '#cccccc',
        '67' : '#ffffff',
        '75' : '#ff0000',
        '83' : '#00ff00',
        '91' : '#0000ff',
        '99' : '#ffff00',
        '107' : '#00ffff',
        '115' : '#ff00ff'
    }
};

BM.Behavior.register( {
    '#CSSGROUP_textTogField:click' : function(e,evt) {
        var a = e.tagName == 'A'        ? e
          : e.parentNode.tagName == 'A' ? e.parentNode
          : null;
        if (!a) return;
        var c = a.className;
        if (c.indexOf('bmcp_list_button') != -1) {//override existing
            BM.FormWidget.updateImageButtons(e);
            BM.CSSedit.clickValueButton(a,evt);
            Event.stop(evt); return false;
        }
        else if (c.indexOf('bmcp_tritoggle') != -1) {
            var a = Event.findElement(evt,'A');
            BM.CSSedit.singleton().changeCSSByField(a.bmField);
            Event.stop(evt); return false;
        }
    },
    '#highlight_element_1:click' : function(e) {
        BM.CSSedit.singleton().updateHighlight();
    },
    '#selector:change' : function(e) {
        BM.CSSedit.singleton().updateSelector();
    },
    '#BM_CSSCONTROLS input:change,#BM_CSSCONTROLS select:change':function(e) {
        BM.CSSedit.singleton().changeCSSByField(e);
    },
    '#BM_CLEAR_ALL_STYLES:click' : function(e,evt) {
        var clearStyles = function() {
            BM.confirmLeave = true; // mark field as dirty
            var css = BM.CSSedit.singleton();
            css.rules = {};
            css.initStyles();
            css.updateSelector();
        }
        new BM.Confirm({
          action:clearStyles,
          message:BM.Lang._("CSSedit:Clear styles?"),
          postMessage : BM.Lang._('CSSedit:Custom styles cleared.')
        });
        Event.stop(evt); return false;
    },
    '#BM_PAGE_PREVIEW:change' : function(e) {
        BM.CSSedit.singleton().changePreview(e);
    }
});
